# Empirical yeast data analysis with McClintock2

Here, we introduce a reproducible pipeline to perform McClintock on a comprehensive WGS dataset containing 1,011 natural yeast isolates ([Peter et al. 2018](https://www.nature.com/articles/s41586-018-0030-5)).

## Setup conda environments as described in [simulation framework](https://github.com/bergmanlab/mcclintock/tree/simulation/simulation#install-mcclintock-20-simulation-system)

## Quick Start
- Sample script to submit the entire empirical analysis as a SLURM cluster job:
```bash
#!/bin/bash
#SBATCH --partition=iob_p
#SBATCH --job-name=realdata
#SBATCH --nodes=1
#SBATCH --tasks-per-node=10
#SBATCH --mem=40G
#SBATCH --time=400:00:00
#SBATCH --mail-user=jc33471@uga.edu
#SBATCH --mail-type=BEGIN,END,FAIL
#SBATCH --output=/scratch/jc33471/mcc_paper/real_peter50_smk/realdata_analysis.out

# McC main repo path
mcc_dir="/home/${USER}/mcclintock"
# McC paper repo path
paper_dir="/home/${USER}/mcclintock2_paper"
# prefix for output
prefix="peter50_smk"
test_dir="/scratch/${USER}/mcc_paper/real_"${prefix}
mkdir -p $test_dir

CONDA_BASE=$(conda info --base)
source ${CONDA_BASE}/etc/profile.d/conda.sh
conda activate mcc_sim 
cd $test_dir
mkdir -p logs/ config/ data/ out/
# absolute path to config file for the entire simulation framework
config=${test_dir}/config/test.json

# prepare the list of samples (SRA accession numbers, merged samples delimited with "_")
samples=`cat ${paper_dir}/empirical/peter_SRA_list.txt`

# create config file
python ${paper_dir}/empirical/make_snakemake_config_empirical.py \
    --mcc ${mcc_dir} \
    --paperdir ${paper_dir} \
    --out ${config} \
    --outdir ${test_dir} \
    --prefix ${prefix} \
    --reference "${mcc_dir}/test/sacCer2.fasta" \
    --consensus "${mcc_dir}/test/sac_cer_TE_seqs.fasta" \
    --locations "${mcc_dir}/test/reference_TE_locations.gff" \
    --taxonomy "${mcc_dir}/test/sac_cer_te_families.tsv" \
    --coverageseq "${paper_dir}/empirical/sacTEcoverage.fa" \
    --samples ${samples} \
    --threads $SLURM_NTASKS

# submit mcc jobs for each sample
snakemake \
    --jobs 200 \
    --use-conda \
    --latency-wait 60 \
    --keep-going \
    --restart-times 3 \
    --snakefile "${paper_dir}/empirical/Snakefile_empirical" \
    --configfile ${config} \
    --cluster-status "${mcc_dir}/simulation/status-sacct-robust.sh" \
    --cluster-cancel 'scancel' \
    --cluster '
        sbatch \
            --partition=batch \
            --parsable \
            --nodes=1 \
            --tasks-per-node={threads} \
            --mem={resources.mem} \
            --time=72:00:00 \
            --mail-user=jc33471@uga.edu \
            --mail-type=END,FAIL \
            --output=logs/{wildcards.sample_name}_slurm.o \
            --error=logs/{wildcards.sample_name}_slurm.e'

# generating plots
snakemake \
    --cores $SLURM_NTASKS \
    --use-conda \
    --configfile ${config} \
    --snakefile "${paper_dir}/empirical/Snakefile_empirical_analysis"
```

## Description on empirical data analysis in the paper
- The sample script of evaluation framework consists of three sections:
  - `empirical/make_snakemake_config_empirical.py`: Python script used to generate a configuration for yeast data analysis.
  - `empirical/Snakefile_empirical`: Snakefile that downloads FASTQ data from SRA, downsamples to 50x fold-coverage, and then performs the complete McClintock2 meta-pipeline. Each sample would be submitted as a separate job to SLURM cluster.
  - `empirical/Snakefile_empirical_analysis`: Snakefile that summarizes and visualizes real data results.

### 1: Creating configuration for empirical data
```
usage: make_snakemake_config_empirical.py [-h] --mcc MCC --paperdir PAPERDIR --out OUT --outdir OUTDIR --reference REFERENCE --consensus CONSENSUS --locations LOCATIONS --taxonomy TAXONOMY --samples SAMPLES [SAMPLES ...] [--coverageseq COVERAGESEQ]
                                          [--prefix PREFIX] [--threads THREADS] [--memory MEMORY]

Create config file in json format for snakemake pipeline that runs empirical data analysis.

optional arguments:
  -h, --help            show this help message and exit

Required:
  --mcc MCC             Path to local McClintock repo. Required.
  --paperdir PAPERDIR   Path to local McClintock2 paper repo. Required.
  --out OUT             File name of the output config file in json. Required.
  --outdir OUTDIR       Output directory for all simulation results. Required.
  --reference REFERENCE
                        [McClintock option] A reference genome sequence in fasta format. Required.
  --consensus CONSENSUS
                        [McClintock option] The consensus sequences of the TEs for the species. Required.
  --locations LOCATIONS
                        [McClintock option] The locations of known TEs in the reference genome in GFF 3 format. Required.
  --taxonomy TAXONOMY   [McClintock option] A tab delimited file with one entry per ID. Required.
  --samples SAMPLES [SAMPLES ...]
                        List of SRA accession numbers to run. Required.

Optional:
  --coverageseq COVERAGESEQ
                        [McClintock option] Sequences of the TEs for coverage module.
  --prefix PREFIX       Prefix of output.

Optional resources:
  --threads THREADS     The number of processors to use for individual cluster jobs. [default = 4]
  --memory MEMORY       The number of memory in 'G' to use for individual cluster jobs. [default = '20G']
```

#### [`empirical/make_snakemake_config_empirical.py`](https://github.com/bergmanlab/mcclintock2_paper/blob/main/empirical/make_snakemake_config_empirical.py) inputs
- Required:
  - `--reference`: A reference genome sequence in FASTA format. 
    - This is the same required input file to run McClintock.
    - A sample for yeast could be found at `<mcc_dir>/test/sacCer2.fasta`.
  - `--consensus`: The consensus sequences of the TEs for the species in FASTA format.
    - This is the same required input file to run McClintock.
    - A sample for yeast could be found at `<mcc_dir>/test/sac_cer_TE_seqs.fasta`.
  - `--locations`: The locations of known TEs in the reference genome in GFF3 format.
    - This is the same optional input file to run McClintock but required for simulation framework.
    - Locations GFF3 could be generated by running `python3 <mcc_dir>/mcclintock.py --make_annotations -r <reference> -c <consensus> -o <output_folder>` 
      - `<output_folder>/<prefix_of_ref>/reference_te_locations/inrefTEs.gff`
    - A sample for yeast could be found at `<mcc_dir>/test/reference_TE_locations.gff`
  - `--taxonomy`: A tab delimited file with one entry per ID.
    - This is the same optional input file to run McClintock but required for simulation framework.
    - Taxonomy TSV could be generated by running `python3 <mcc_dir>/mcclintock.py --make_annotations -r <reference> -c <consensus> -o <output_folder>`
      - `<output_folder>/<prefix_of_ref>/te_taxonomy/taxonomy.tsv`
    - A sample for yeast could be found at `<mcc_dir>/test/sac_cer_te_families.tsv`
  - `--samples`: Space delimited list of SRA accession numbers to run. One sample could be merged with multiple accessions, which need to be separated with `_` (e.g. `ERR1308992_ERR1309161`).
  - `--mcc`: Absolute path to local McClintock repo.
  - `--paperdir`: Absolute path to local McClintock paper (this) repo.
  - `--out`: File name for the output config file in JSON.
  - `--outdir`: Output directory for the entire simulation framework results.
- Optional:
  - General optional parameters:
    - `--coverageseq`: TE sequences as input for coverage module in FASTA.
      - This is the same optional input file to run McClintock.
      - Default: Same as `--consensus`.
      - Note: Sequence names must not contain special characters of `; & ( ) | * ? [ ] ~ { } < ! ^ " , \  $ / + - #`
      - A sample could be found [`empirical/sacTEcoverage.fa`](https://github.com/bergmanlab/mcclintock2_paper/blob/main/empirical/sacTEcoverage.fa) derived from [`yeast/doc/misc/sacCerParTE.fa`](https://github.com/bergmanlab/yeast/blob/master/doc/misc/sacCerParTE.fa)
    - `--prefix`: Prefix for several output files. 
      - Default: "empirical".
  - Parameters for computing resources:
    - `--threads`: Number of processors to run simulater and McClintock for each replicate.
      - Default: 4.
    - `--memory`: The amount of memory to run simulater and McClintock for each replicate in G.
      - Default: 20G.
- A sample config file in json could be found at [`empirical/cfg_empirical.json`](https://github.com/bergmanlab/mcclintock2_paper/blob/main/empirical/cfg_empirical.json).

### 2: Submitting individual cluster jobs for samples.
- Firstly, short-read WGS data is downloaded from NCBI SRA based on the provided accession numbers. Secondly, all samples are down-sampled to 50x if the original coverage is greater, which is based on our simulation results that 50x is sufficient to optimize performance of component methods. Thirdly, the complete McClintock2 pipeline is performed.
- Each sample would be submitted as a separate cluster job that could run in parallel.
- Run Snakefile [`empirical/Snakefile_empirical`](https://github.com/bergmanlab/mcclintock2_paper/blob/main/empirical/Snakefile_empirical) along with the config file generated above:
```
snakemake \
    --jobs 200 \
    --use-conda \
    --latency-wait 60 \
    --keep-going \
    --restart-times 3 \
    --snakefile "${paper_dir}/empirical/Snakefile_empirical" \
    --configfile ${config} \
    --cluster '
        sbatch \
            --partition=batch \
            --nodes=1 \
            --tasks-per-node={threads} \
            --mem={resources.mem} \
            --time=72:00:00 \
            --mail-user=jc33471@uga.edu \
            --mail-type=END,FAIL \
            --output=logs/{wildcards.sample_name}_slurm.o \
            --error=logs/{wildcards.sample_name}_slurm.e'
```
- Parameters for `snakemake` command are identical as [simulation framework](https://github.com/bergmanlab/mcclintock/tree/simulation/simulation#2-submitting-individual-cluster-jobs-for-simulation-replicates).

### 3: Summarizing and visualizing empirical results.
- Run Snakefile [`empirical/Snakefile_empirical_analysis`](https://github.com/bergmanlab/mcclintock2_paper/blob/main/empirical/Snakefile_empirical_analysis) along with the config file generated above:
```
snakemake \
    --cores $SLURM_NTASKS \
    --use-conda \
    --configfile ${config} \
    --forcerun "all" \
    --snakefile "${paper_dir}/empirical/Snakefile_empirical_analysis"
```
- Output files include:
  - `<outdir>/r_vis/realtsd_TY1.pdf`: TSD length distribution predicted by split-read methods for each TE family.
  - `<outdir>/r_vis/nonref_<prefix>_boxplot.pdf`: Numbers of non-reference predictions made by 12 component methods across samples.
  - `<outdir>/r_vis/supp_cov_<prefix>_boxplot.pdf`: Numbers of non-reference predictions made by coverage module across samples.
  - `<outdir>/r_vis/method_correlation_<prefix>.pdf`: Correlation across 12 component methods in all non-reference predictions.
  - `<outdir>/r_vis/tRNAtable_bar_<prefix>.pdf`: Number of non-reference predictions found near tRNA and non-tRNA sites in all real yeast samples, across 12 component methods and TE families.
  - `<outdir>/r_vis/nuc_relocate2.pdf`: Distribution of Ty insertion relative to tRNA TSS and its association with nucleosome occupancy in natural yeast genomes from RelocaTE2.
  - `<outdir>/r_vis/supp_old_density.pdf` and `<outdir>/r_vis/supp_new_density.pdf`: Distribution of Ty insertion relative to tRNA TSS and its association with nucleosome occupancy in natural yeast genomes from 6 original methods and 6 newly-added methods, respectively.
