import sys
import os
import json
import argparse
import re

def main():

    args = parse_args()

    config = {}
    

    ## parameters for output path
    config["out"] = {
        "prefix": args.prefix,
        "outdir": args.outdir
    }

    ## parameter for mcc installation
    config["mccdir"] = args.mcc

    ## parameter for mcc2 paper
    config["paperdir"] = args.paperdir

    ## parameter for samples
    config['samples'] = args.samples

    ## parameters for simulation script inputs
    config["in"] = {
        "ref": args.reference,
        "consensus": args.consensus,
        "gff": args.locations,
        "tax": args.taxonomy,
        "covseq":args.coverageseq,
        "script": args.paperdir+"/empirical/empirical_merge_down50x_smk.sh"
    }

    ## paths for analysis scripts
    config["analysis"] = {
        "r_vis_data": args.paperdir+"/empirical/empirical_r_vis_data.R",
        "r_vis_plots": args.paperdir+"/empirical/empirical_r_vis_plots.R"
    }

    ## parameters for cluster job resources/envs
    config["resources"] = {
        "threads": args.threads,
        "mem": args.memory,
        "condaenv": "mcclintock",
        "renv": "mcc_analysis"
    }

    
    with open(args.out, "w") as conf:
        json.dump(config, conf, indent=4)





def parse_args():
    parser = argparse.ArgumentParser(prog='make_snakemake_config_empirical.py', description="Create config file in json format for snakemake pipeline that runs empirical data analysis.")

    ## required ##
    essential_args = parser.add_argument_group("Required")
    essential_args.add_argument("--mcc", type=str, help="Path to local McClintock repo. Required.", required=True)
    essential_args.add_argument("--paperdir", type=str, help="Path to local McClintock2 paper repo. Required.", required=True)
    essential_args.add_argument("--out", type=str, help="File name of the output config file in json. Required.", required=True)
    essential_args.add_argument("--outdir", type=str, help="Output directory for all simulation results. Required.", required=True)
    essential_args.add_argument("--reference", type=str, help="[McClintock option] A reference genome sequence in fasta format. Required.", required=True)
    essential_args.add_argument("--consensus", type=str, help="[McClintock option] The consensus sequences of the TEs for the species. Required.", required=True)
    essential_args.add_argument("--locations", type=str, help="[McClintock option] The locations of known TEs in the reference genome in GFF 3 format. Required.", required=True)
    essential_args.add_argument("--taxonomy", type=str, help="[McClintock option] A tab delimited file with one entry per ID. Required.", required=True)
    essential_args.add_argument("--samples", type=str, nargs='+', help="List of SRA accession numbers to run. Required.", required=True)


    ## optional ##
    optional_args = parser.add_argument_group('Optional')
    optional_args.add_argument("--coverageseq", type=str, help="[McClintock option] Sequences of the TEs for coverage module.", required=False)
    optional_args.add_argument("--prefix", type=str, help="Prefix of output.", default="empirical", required=False)
    ## resources options ##
    resources_args = parser.add_argument_group('Optional resources')
    resources_args.add_argument("--threads", type=int, help="The number of processors to use for individual cluster jobs. [default = 4]", default=4, required=False)
    resources_args.add_argument("--memory", type=str, help="The number of memory in 'G' to use for individual cluster jobs. [default = '20G']", default="20G", required=False)


    args = parser.parse_args()

    ## required ##
    # parse mcc install path
    args.mcc = os.path.abspath(args.mcc)

    # parse mcc paper path
    args.paperdir = os.path.abspath(args.paperdir)
    
    # parse output path for the json config
    args.out = os.path.abspath(args.out)

    # parse output folder of simulation results
    args.outdir = os.path.abspath(args.outdir)

    # parse reference
    args.reference = os.path.abspath(args.reference)

    # parse consensus
    args.consensus = os.path.abspath(args.consensus)

    # parse locations
    args.locations = os.path.abspath(args.locations)

    # parse taxonomy
    args.taxonomy = os.path.abspath(args.taxonomy)

    # parse samples
    for i in args.samples:
        if re.search("(^SRR)|(^ERR)", i) is None:
            sys.stderr.write("Provide available SRA accession numbers")
            sys.exit(1)

    ## optional ##
    # parse coverageseq
    if args.coverageseq is None:
        args.coverageseq = args.consensus

    # parse threads
    args.threads = int(args.threads)
    
    # parse memory
    if re.search("G$", args.memory) is None:
        sys.stderr.write("Only memory in 'G' is allowed for --memory option, i.e, '20G'")
        sys.exit(1)
    
    return args


if __name__ == "__main__":
    main()

