# Running time trial performance 
- This analysis aims to investigate the improvement on running time/CPU efficiency in McClintock 2.
- Note that this analysis is highly depending on the cluster availability. Please make sure the cluster network is available and stable during the job.

## Installation
### Setup McClintock 2
- See main repo: https://github.com/bergmanlab/mcclintock/blob/master/README.md#-getting-started

### Setup Simulation framework
- See main repo: https://github.com/bergmanlab/mcclintock/blob/simulation/simulation/README.md

## Performance analysis
### Running McClintock jobs
```bash
#!/bin/bash
#SBATCH --partition=bergman_p
#SBATCH --job-name=mcc-performance-100
#SBATCH --nodes=1
#SBATCH --tasks-per-node=10
#SBATCH --mem=20G
#SBATCH --time=400:00:00
#SBATCH --mail-user=jc33471@uga.edu
#SBATCH --mail-type=END,FAIL
#SBATCH --output=/scratch/jc33471/mcc_paper/performance_mar/performance.out

for cov in 50 100; do
    mcc_dir="/home/jc33471/mcclintock"
    run_dir="/scratch/jc33471/mcc_paper/performance_mar/${cov}"
    data_dir="/home/jc33471/mcclintock/test"
    script_dir=$run_dir/scripts
    out_dir=$run_dir/out
    seff_dir=$run_dir/seff
    log_dir=$run_dir/log
    mkdir -p ${run_dir}
    mkdir -p ${script_dir} ${out_dir} ${log_dir} ${seff_dir}

    # download test data
    cd $mcc_dir
    if [[ ! -e $data_dir/SRR800842_1.fastq ]]; then
        python3 test/download_test_data.py
        gunzip $data_dir/SRR800842_1.fastq.gz
        gunzip $data_dir/SRR800842_2.fastq.gz
    fi 

    # downsample test data
    module load seqtk/1.3-GCC-8.3.0
    fq1=$data_dir/SRR800842_${cov}x_1.fastq
    fq2=$data_dir/SRR800842_${cov}x_2.fastq
    num_read=$((12000000 * cov / 2 / 101))
    seqtk sample -s233 $data_dir/SRR800842_1.fastq ${num_read} > $fq1
    seqtk sample -s233 $data_dir/SRR800842_2.fastq ${num_read} > $fq2
    module unload seqtk/1.3-GCC-8.3.0

    cd ${run_dir}
    # for cores in 1 5 10 15; do
    mem="20G"
    JOBID=1
    jobid_list=${seff_dir}/jobid.list; > ${jobid_list}
    for rep in {1..5}; do
        for cores in 1 5 10 15 20 25 30; do
            # submit mcc1 job
            qsub=${script_dir}/mcc1_core${cores}_rep${rep}.sh

            echo '#!/bin/bash' > $qsub
            echo "#SBATCH --job-name=mcc1_core${cores}_rep${rep}" >> $qsub
            echo "#SBATCH --partition=bergman_p" >> $qsub
            echo "#SBATCH --nodes=1" >> $qsub
            echo "#SBATCH --tasks-per-node=${cores}" >> $qsub
            echo "#SBATCH --mem=${mem}" >> $qsub
            echo "#SBATCH --time=100:00:00" >> $qsub
            echo "#SBATCH --mail-type=END,FAIL" >> $qsub
            echo "#SBATCH --mail-user=${SLURM_JOB_USER}@uga.edu" >> $qsub
            echo "#SBATCH --output=$log_dir/mcc1_core${cores}_rep${rep}.oe" >> $qsub
            echo "#SBATCH --export=NONE" >> $qsub
            echo "#SBATCH --nodelist=d1-6" >> $qsub

            echo "starttime=\`date +%s\`" >> $qsub
            echo "bash ~/mcclintock_1.0/mcclintock/mcclintock.sh  \\" >> $qsub
            echo "-r $data_dir/sacCer2.fasta \\" >> $qsub
            echo "-c $data_dir/sac_cer_TE_seqs.fasta \\" >> $qsub
            echo "-g $data_dir/reference_TE_locations.gff \\" >> $qsub
            echo "-t $data_dir/sac_cer_te_families.tsv \\" >> $qsub
            echo "-1 $fq1 \\" >> $qsub
            echo "-2 $fq2 \\" >> $qsub
            echo "-m ngs_te_mapper,relocate,temp,retroseq,popoolationte,te-locate \\" >> $qsub
            echo "-p $cores \\" >> $qsub
            # echo "-i \\" >> $qsub
            echo "-o $out_dir/mcc1_core${cores}_rep${rep}" >> $qsub
            echo "endtime=\`date +%s\`" >> $qsub
            echo "time=\$((endtime-starttime))" >> $qsub
            echo "echo \$time" >> $qsub

            rm $log_dir/mcc1_core${cores}_rep${rep}.oe
            rm -rf $out_dir/mcc1_core${cores}_rep${rep}
            JOBID=$(sbatch --parsable --dependency=afterok:${JOBID} $qsub)
            echo "${JOBID}" >> ${jobid_list}

            # submit mcc2 job
            qsub=${script_dir}/mcc2_core${cores}_rep${rep}.sh

            echo '#!/bin/bash' > $qsub
            echo "#SBATCH --job-name=mcc2_core${cores}_rep${rep}" >> $qsub
            echo "#SBATCH --partition=bergman_p" >> $qsub
            echo "#SBATCH --nodes=1" >> $qsub
            echo "#SBATCH --tasks-per-node=${cores}" >> $qsub
            echo "#SBATCH --mem=${mem}" >> $qsub
            echo "#SBATCH --time=100:00:00" >> $qsub
            echo "#SBATCH --mail-type=END,FAIL" >> $qsub
            echo "#SBATCH --mail-user=${SLURM_JOB_USER}@uga.edu" >> $qsub
            echo "#SBATCH --output=$log_dir/mcc2_core${cores}_rep${rep}.oe" >> $qsub
            echo "#SBATCH --export=NONE" >> $qsub
            echo "#SBATCH --nodelist=d1-6" >> $qsub

            CONDA_BASE=$(conda info --base)
            echo "CONDA_BASE=$(conda info --base)" >> $qsub
            echo "source ${CONDA_BASE}/etc/profile.d/conda.sh" >> $qsub
            echo "conda activate mcclintock" >> $qsub

            echo "starttime=\`date +%s\`" >> $qsub
            echo "python3 $mcc_dir/mcclintock.py  \\" >> $qsub
            echo "-r $data_dir/sacCer2.fasta \\" >> $qsub
            echo "-c $data_dir/sac_cer_TE_seqs.fasta \\" >> $qsub
            echo "-g $data_dir/reference_TE_locations.gff \\" >> $qsub
            echo "-t $data_dir/sac_cer_te_families.tsv \\" >> $qsub
            echo "-1 $fq1 \\" >> $qsub
            echo "-2 $fq2 \\" >> $qsub
            echo "-m ngs_te_mapper,relocate,temp,retroseq,popoolationte,te-locate \\" >> $qsub
            echo "-p $cores \\" >> $qsub
            echo "--keep_intermediate minimal \\" >> $qsub
            echo "-o $out_dir/mcc2_core${cores}_rep${rep}" >> $qsub

            echo "endtime=\`date +%s\`" >> $qsub
            echo "time=\$((endtime-starttime))" >> $qsub
            echo "echo \$time" >> $qsub
            
            rm $log_dir/mcc2_core${cores}_rep${rep}.oe
            rm -rf $out_dir/mcc2_core${cores}_rep${rep}
            JOBID=$(sbatch --parsable --dependency=afterok:${JOBID} $qsub)
            echo "${JOBID}" >> ${jobid_list}

            # submit 12-method jobs
            qsub=${script_dir}/mcc2all_core${cores}_rep${rep}.sh

            echo '#!/bin/bash' > $qsub
            echo "#SBATCH --job-name=mcc2all_core${cores}_rep${rep}" >> $qsub
            echo "#SBATCH --partition=bergman_p" >> $qsub
            echo "#SBATCH --nodes=1" >> $qsub
            echo "#SBATCH --tasks-per-node=${cores}" >> $qsub
            echo "#SBATCH --mem=${mem}" >> $qsub
            echo "#SBATCH --time=100:00:00" >> $qsub
            echo "#SBATCH --mail-type=END,FAIL" >> $qsub
            echo "#SBATCH --mail-user=${SLURM_JOB_USER}@uga.edu" >> $qsub
            echo "#SBATCH --output=$log_dir/mcc2all_core${cores}_rep${rep}.oe" >> $qsub
            echo "#SBATCH --export=NONE" >> $qsub
            echo "#SBATCH --nodelist=d1-6" >> $qsub

            CONDA_BASE=$(conda info --base)
            echo "CONDA_BASE=$(conda info --base)" >> $qsub
            echo "source ${CONDA_BASE}/etc/profile.d/conda.sh" >> $qsub
            echo "conda activate mcclintock" >> $qsub

            echo "starttime=\`date +%s\`" >> $qsub
            echo "python3 $mcc_dir/mcclintock.py  \\" >> $qsub
            echo "-r $data_dir/sacCer2.fasta \\" >> $qsub
            echo "-c $data_dir/sac_cer_TE_seqs.fasta \\" >> $qsub
            echo "-g $data_dir/reference_TE_locations.gff \\" >> $qsub
            echo "-t $data_dir/sac_cer_te_families.tsv \\" >> $qsub
            echo "-1 $fq1 \\" >> $qsub
            echo "-2 $fq2 \\" >> $qsub
            echo "-m ngs_te_mapper,ngs_te_mapper2,relocate,relocate2,temp,temp2,retroseq,popoolationte,popoolationte2,te-locate,teflon,tebreak \\" >> $qsub
            echo "-p $cores \\" >> $qsub
            echo "--keep_intermediate minimal \\" >> $qsub
            echo "-o $out_dir/mcc2all_core${cores}_rep${rep}" >> $qsub

            echo "endtime=\`date +%s\`" >> $qsub
            echo "time=\$((endtime-starttime))" >> $qsub
            echo "echo \$time" >> $qsub
            
            rm $log_dir/mcc2all_core${cores}_rep${rep}.oe
            rm -rf $out_dir/mcc2all_core${cores}_rep${rep}
            JOBID=$(sbatch --parsable --dependency=afterok:${JOBID} $qsub)
            echo "${JOBID}" >> ${jobid_list}

        done
    done
done
```

### Getting job status after all McClintock jobs are done
```bash
for cov in 50 100; do
    mcc_dir="/home/jc33471/mcclintock"
    run_dir="/scratch/jc33471/mcc_paper/performance_mar/${cov}"
    data_dir="/home/jc33471/mcclintock/test"
    script_dir=$run_dir/scripts
    out_dir=$run_dir/out
    seff_dir=$run_dir/seff
    log_dir=$run_dir/log

    ## Slurm Job Efficiency Report
    jobid_list=${seff_dir}/jobid.list
    printf "Job ID,Cluster,User/Group,State,Cores,CPU Utilized,CPU Efficiency,Job Wall-clock time,Memory Utilized,Memory Efficiency\n" > ${seff_dir}/seff_sum.csv
    while read i; do
        seff ${i} > ${seff_dir}/${i}.seff
        cat ${seff_dir}/${i}.seff |\
            sed '/Nodes: 1/d' |\
            sed 's/: /,/g' |\
            datamash transpose --field-separator=',' --output-delimiter=',' --no-strict |\
            sed 1d > ${seff_dir}/${i}.csv
        cat ${seff_dir}/${i}.csv >> ${seff_dir}/seff_sum.csv
    done < ${jobid_list}

    ## summarize speed (old-fashioned)
    printf "version,cores,rep,time\n" > ${run_dir}/speed_sum.csv
    for log in `ls -1 $log_dir/*`; do
        run=`basename ${log} .oe`
        version=`echo ${run} | cut -f1 -d_`
        cores=`echo ${run} | cut -f2 -d_ | sed s/core//g`
        rep=`echo ${run} | cut -f3 -d_`
        time=`tail -n1 ${log}`
        printf "${version},${cores},${rep},${time}\n" >> ${run_dir}/speed_sum.csv
    done

done
```

### Visualizing in curves
```bash
conda activate mcc_analysis
paper_dir="/home/${USER}/mcclintock2_paper"
performance_dir="/scratch/${USER}/mcc_paper/performance_mar"
Rscript --vanilla ${paper_dir}/performance/sim_performance_fig.R ${performance_dir} 
```