require(tools)
library(ggplot2)
library(plyr); library(dplyr)
library(tidyr)
library(Rmisc)
library(ggpubr)
library(reshape2)
# library(optparse)
pdf(NULL)

# parse arguments
args = commandArgs(trailingOnly=TRUE)
if (length(args) != 1) {
  stop("Usage: Rscript --vanilla sim_performance_fig.R <performance_dir>", call.=FALSE)
} else if (length(args) == 1) {
  run_dir <- args[1]
}

result_dir <- paste0(run_dir, "/r_vis")
dir.create(result_dir, showWarnings = FALSE, recursive = TRUE)

covs = c(50, 100)

############################ speed evaluation for each cov #############################
for (cov in covs){
  speed_sum <- paste0(run_dir, "/", cov, "/speed_sum.csv")
  speed <- read.csv(speed_sum, stringsAsFactors = F)
  setcolors <- c(
    "mcc1" = "#E69F00",
    "mcc2" = "#56B4E9",
    "mcc2all" = "#0072B2"
  )

  # convert sec to min
  speed$time <- speed$time/60
  speedsum <- summarySE(speed, measurevar="time", groupvars=c("version","cores"))
  linesize <- 2
  p <- speedsum %>%  ggplot( aes(x=as.factor(cores), y=time, group=version, color=version)) +
    geom_line(size = linesize) +
    geom_errorbar(aes(ymin=as.numeric(time)-sd, ymax=as.numeric(time)+sd), width=.1, size = 1) +
    theme_bw() +
    theme(legend.position = c(0.85,0.9), legend.background = element_rect(linetype = 1, size = 0.3, color = "gray"),
          text = element_text(size = 15)) +
    labs(x = "Number of CPU cores", y = "Run time (in minutes)") +
    scale_color_manual(values = setcolors, name = NULL, labels = c("McClintock 1 (6 methods)", "McClintock 2 (6 methods)", "McClintock 2 (12 methods)")) 
  ggsave(plot = p, paste0(result_dir, "/speed_evaluation_", cov, "x.pdf"), device = "pdf", width = 10, height = 8, units = "in", dpi = 320)

}

############################ speed evaluation for all covs #############################
# set colors for mcc run options
setcolors <- c(
  "mcc1" = "#E69F00",
  "mcc2" = "#56B4E9",
  "mcc2all" = "#0072B2"
)
colorlabels <- c(
  "mcc1" = "McClintock 1 (6 methods)",
  "mcc2" = "McClintock 2 (6 methods)",
  "mcc2all" = "McClintock 2 (12 methods)"
)
# set line types for 
setlinetype <- c(
  "50" = "solid",
  "100" = "solid"
)
linetypelabels <- c(
  "50" = "50X",
  "100" = "100X"
)
# combine covs
speed_sum50 <- paste0(run_dir, "/50/speed_sum.csv")
speed50 <- read.csv(speed_sum50, stringsAsFactors = F) %>% mutate(cov = 50)
speed_sum100 <- paste0(run_dir, "/100/speed_sum.csv")
speed100 <- read.csv(speed_sum100, stringsAsFactors = F) %>% mutate(cov = 100)
speed_all <- rbind(speed50, speed100)

# convert sec to min
speed_all$time <- speed_all$time/60
speedsum <- summarySE(speed_all, measurevar="time", groupvars=c("version","cores","cov"))
linesize <- 1
p <- speedsum %>%  ggplot( aes(x=as.factor(cores), y=time, group=interaction(version,cov), color=as.factor(version))) +
  geom_line(size = linesize) +
  geom_errorbar(aes(ymin=as.numeric(time)-sd, ymax=as.numeric(time)+sd), width=.1, size = 1) +
  theme_bw() +
  theme( legend.position = "bottom",
        text = element_text(size = 15)) +
  labs(x = "Number of CPU cores", y = "Run time (in minutes)") +
  scale_color_manual(values = setcolors, name = NULL, labels = colorlabels) +
  #scale_linetype_manual(values = setlinetype, name = NULL, labels = linetypelabels) +
  facet_wrap(vars(cov), labeller = as_labeller(linetypelabels))
ggsave(plot = p, paste0(result_dir, "/speed_evaluation_covs.pdf"), device = "pdf", width = 10, height = 6, units = "in", dpi = 320)

############################ other performance stat for all covs #############################
seff50 <- read.csv(paste0(run_dir, "/50/seff/seff_sum.csv"), stringsAsFactors = F) %>%
  mutate(cov = 50, version = rep(c("mcc1","mcc2","mcc2all"), times = 35), rep = rep(c("rep1","rep2","rep3","rep4","rep5"), each = 21))
seff100 <- read.csv(paste0(run_dir, "/100/seff/seff_sum.csv"), stringsAsFactors = F) %>%
  mutate(cov = 100, version = rep(c("mcc1","mcc2","mcc2all"), times = 35), rep = rep(c("rep1","rep2","rep3","rep4","rep5"), each = 21))
seff_all <- rbind(seff50, seff100) %>%
  mutate(cpu_efficiency = as.numeric(gsub("% of.*$", "", CPU.Efficiency, perl = T))) %>%
  mutate(mem_efficiency = as.numeric(gsub("% of.*$", "", Memory.Efficiency, perl = T))) %>%
  mutate(cores = Cores)

# CPU Efficiency
cpu_sum <- seff_all %>% select(c(cores,cov,version,rep,cpu_efficiency,mem_efficiency)) %>%
  summarySE(measurevar="cpu_efficiency", groupvars=c("version","cores","cov"))

linesize <- 1
p <- cpu_sum %>%  ggplot( aes(x=as.factor(cores), y=cpu_efficiency, group=interaction(version,cov), color=as.factor(version))) +
  geom_line(size = linesize) +
  geom_errorbar(aes(ymin=as.numeric(cpu_efficiency)-sd, ymax=as.numeric(cpu_efficiency)+sd), width=.1, size = 1) +
  theme_bw() +
  theme( legend.position = "bottom",
         text = element_text(size = 15)) +
  labs(x = "Number of CPU cores", y = "CPU Efficiency (%)") +
  scale_color_manual(values = setcolors, name = NULL, labels = colorlabels) +
  #scale_linetype_manual(values = setlinetype, name = NULL, labels = linetypelabels) +
  facet_wrap(vars(cov), labeller = as_labeller(linetypelabels))
ggsave(plot = p, paste0(result_dir, "/cpu_efficiency_covs.pdf"), device = "pdf", width = 10, height = 6, units = "in", dpi = 320)

# get legend in a separate file
legend <- ggpubr::get_legend(p)
ggpubr::as_ggplot(legend)
ggsave(paste0(result_dir,"/legend_perform.pdf"), device = "pdf", width = 10, height = 0.5, units = "in", dpi = 320)

# # Memory Efficiency
# mem_sum <- seff_all %>% select(c(cores,cov,version,rep,cpu_efficiency,mem_efficiency)) %>%
#   summarySE(measurevar="mem_efficiency", groupvars=c("version","cores","cov"))

# linesize <- 1
# p <- mem_sum %>%  ggplot( aes(x=as.factor(cores), y=mem_efficiency, group=interaction(version,cov), color=as.factor(version))) +
#   geom_line(size = linesize) +
#   geom_errorbar(aes(ymin=as.numeric(mem_efficiency)-sd, ymax=as.numeric(mem_efficiency)+sd), width=.1, size = 1) +
#   theme_bw() +
#   theme( legend.position = "bottom",
#          text = element_text(size = 15)) +
#   labs(x = "Number of CPU cores", y = "Memory Efficiency (% Out of 20 GB)") +
#   scale_color_manual(values = setcolors, name = NULL, labels = colorlabels) +
#   #scale_linetype_manual(values = setlinetype, name = NULL, labels = linetypelabels) +
#   facet_wrap(vars(cov), labeller = as_labeller(linetypelabels))
# ggsave(plot = p, paste0(result_dir, "/mem_efficiency_covs.pdf"), device = "pdf", width = 10, height = 6, units = "in", dpi = 320)
