# Running all simulation batches for McC2 paper
- Sim 1: Simulation 1 aimed to test the new simulation code and replicate previous results reported in the original McClintock paper [(Nelson, Linheiro and Bergman (2017) G3 7:2763-2778)](https://academic.oup.com/g3journal/article/7/8/2763/6031520) using predictions from McClintock 1 based on simulated WGS datasets generated by the new simulation system available in McClintock 2.
- Sim 2: Simulation 2 aimed to determine the accuracy of the McClintock re-implementation for the six original components available in McClintock 1 using predictions from McClintock 2
- Sim 3: Simulation 3 aimed to evaluate the ability of all 12 components in McClintock 2 to detect TE insertions in their biologically-realistic locations in the *Saccharomyces cerevisiae* genome (tRNA promoter regions) over a range of WGS coverages.
- Sim 4: Simulation 4 aimed to evaluate the performance of all 12 components in McClintock 2 to detect TE insertions in random, unique genomic regions over a range of WGS coverages.


## Installation
### Setup original McClintock
```bash
mkdir ~/mcclintock_1.0
cd ~/mcclintock_1.0
git clone git@github.com:bergmanlab/mcclintock.git
cd mcclintock

# switch to the mcc 1.0 branch
git checkout original

# install mcc 1.0
conda activate base
bash env_install.sh
conda deactivate
conda activate MCCLINTOCK_main # make sure unzip and patch are available
bash install.sh
conda deactivate
```

### Setup McClintock 2
- See main repo: https://github.com/bergmanlab/mcclintock/blob/master/README.md#-getting-started

### Setup Simulation framework
- See main repo: https://github.com/bergmanlab/mcclintock/blob/simulation/simulation/README.md

## Running Simulation
### Simulation 1
```bash
#!/bin/bash
#SBATCH --partition=iob_p
#SBATCH --job-name=simulation_batch1
#SBATCH --nodes=1
#SBATCH --tasks-per-node=10
#SBATCH --mem=40G
#SBATCH --time=400:00:00
#SBATCH --mail-user=jc33471@uga.edu
#SBATCH --mail-type=BEGIN,END,FAIL
#SBATCH --output=/scratch/jc33471/mcc_paper/simulation_smk/simulation_1.out

total_dir="/scratch/jc33471/mcc_paper/simulation_smk"
mkdir -p ${total_dir}
mcc_dir="/home/${USER}/mcclintock"
mcc1_dir="/home/${USER}/mcclintock_1.0/mcclintock/"

CONDA_BASE=$(conda info --base)
source ${CONDA_BASE}/etc/profile.d/conda.sh
conda activate mcc_sim

############### simulation 1 ###############
sim_dir=${total_dir}/sim_1
mkdir -p $sim_dir

cd $sim_dir
mkdir -p logs/ config/
# Create config for each replicate
python ${mcc_dir}/simulation/archive/make_old_sim_config.py \
    ${mcc_dir}/test/tRNA_coordinates.gff \
    ${mcc1_dir} ${mcc_dir}

# absolute path to config file for the entire simulation framework
config=${sim_dir}/config/sim_1.json

# make json config for snakemake
python ${mcc_dir}/simulation/make_snakemake_config.py \
    --family "TY1" "TY2" "TY3" "TY4" \
    --bed "${mcc_dir}/simulation/yeast/200-195_tRNA_targets.bed" \
        "${mcc_dir}/simulation/yeast/200-195_tRNA_targets.bed" \
        "${mcc_dir}/simulation/yeast/17-12_tRNA_targets.bed" \
        "${mcc_dir}/simulation/yeast/200-195_tRNA_targets.bed" \
    --mcc ${mcc_dir} \
    --out ${config} \
    --outdir ${sim_dir} \
    --reference "${mcc_dir}/test/sacCer2.fasta" \
    --consensus "${mcc_dir}/test/sac_cer_TE_seqs.fasta" \
    --locations "${mcc_dir}/test/reference_TE_locations.gff" \
    --taxonomy "${mcc_dir}/test/sac_cer_te_families.tsv" \
    --covrange 100 --numrep 299 \
    --oldsim --mcc_version 1 --sim "wgsim"

chmod a+x ${mcc_dir}/simulation/status-sacct-robust.sh
snakemake \
    --jobs 200 \
    --use-conda \
    --latency-wait 60 \
    --keep-going \
    --restart-times 0 \
    --snakefile "${mcc_dir}/simulation/Snakefile_sim" \
    --configfile ${config} \
    --cluster-status "${mcc_dir}/simulation/status-sacct-robust.sh" \
    --cluster-cancel 'scancel' \
    --cluster '
        sbatch \
            --partition=batch \
            --nodes=1 \
            --tasks-per-node={threads} \
            --mem={resources.mem} \
            --time=12:00:00 \
            --parsable \
            --mail-user=jc33471@uga.edu \
            --mail-type=END,FAIL \
            --output=logs/{wildcards.cov}x_{wildcards.strand}_rep{wildcards.rep}.o \
            --error=logs/{wildcards.cov}x_{wildcards.strand}_rep{wildcards.rep}.e'

snakemake \
    --cores $SLURM_NTASKS \
    --use-conda \
    --configfile ${config} \
    --snakefile "${mcc_dir}/simulation/Snakefile_analysis"

rm ${sim_dir}/*/data/*/*.fastq
```

### Simulation 2
```bash
#!/bin/bash
#SBATCH --partition=iob_p
#SBATCH --job-name=simulation_batch2
#SBATCH --nodes=1
#SBATCH --tasks-per-node=10
#SBATCH --mem=40G
#SBATCH --time=400:00:00
#SBATCH --mail-user=jc33471@uga.edu
#SBATCH --mail-type=BEGIN,END,FAIL
#SBATCH --output=/scratch/jc33471/mcc_paper/simulation_smk/simulation_2.out

total_dir="/scratch/jc33471/mcc_paper/simulation_smk"
mkdir -p ${total_dir}
mcc_dir="/home/${USER}/mcclintock"
mcc1_dir="/home/${USER}/mcclintock_1.0/mcclintock/"

CONDA_BASE=$(conda info --base)
source ${CONDA_BASE}/etc/profile.d/conda.sh
conda activate mcc_sim

############### simulation 2 ###############
sim_dir=${total_dir}/sim_2
mkdir -p $sim_dir

cd $sim_dir
mkdir -p logs/ config/
# Create config for each replicate
python ${mcc_dir}/simulation/archive/make_old_sim_config.py \
    ${mcc_dir}/test/tRNA_coordinates.gff \
    ${mcc1_dir} ${mcc_dir}

# absolute path to config file for the entire simulation framework
config=${sim_dir}/config/sim_2.json

# make json config for snakemake
python ${mcc_dir}/simulation/make_snakemake_config.py \
    --family "TY1" "TY2" "TY3" "TY4" \
    --bed "${mcc_dir}/simulation/yeast/200-195_tRNA_targets.bed" \
        "${mcc_dir}/simulation/yeast/200-195_tRNA_targets.bed" \
        "${mcc_dir}/simulation/yeast/17-12_tRNA_targets.bed" \
        "${mcc_dir}/simulation/yeast/200-195_tRNA_targets.bed" \
    --mcc ${mcc_dir} \
    --out ${config} \
    --outdir ${sim_dir} \
    --reference "${mcc_dir}/test/sacCer2.fasta" \
    --consensus "${mcc_dir}/test/sac_cer_TE_seqs.fasta" \
    --locations "${mcc_dir}/test/reference_TE_locations.gff" \
    --taxonomy "${mcc_dir}/test/sac_cer_te_families.tsv" \
    --covrange 100 --numrep 299 \
    --oldsim --mcc_version 2 --sim "wgsim"

snakemake \
    --jobs 200 \
    --use-conda \
    --latency-wait 60 \
    --keep-going \
    --restart-times 3 \
    --snakefile "${mcc_dir}/simulation/Snakefile_sim" \
    --configfile ${config} \
    --cluster '
        sbatch \
            --partition=batch \
            --nodes=1 \
            --tasks-per-node={threads} \
            --mem={resources.mem} \
            --time=12:00:00 \
            --mail-user=jc33471@uga.edu \
            --mail-type=END,FAIL \
            --output=logs/{wildcards.cov}x_{wildcards.strand}_rep{wildcards.rep}.o \
            --error=logs/{wildcards.cov}x_{wildcards.strand}_rep{wildcards.rep}.e'

snakemake \
    --cores $SLURM_NTASKS \
    --use-conda \
    --configfile ${config} \
    --snakefile "${mcc_dir}/simulation/Snakefile_analysis"

rm ${sim_dir}/*/data/*/*.fastq
```


### Simulation 3
```bash
#!/bin/bash
#SBATCH --partition=iob_p
#SBATCH --job-name=simulation_batch3
#SBATCH --nodes=1
#SBATCH --tasks-per-node=10
#SBATCH --mem=40G
#SBATCH --time=400:00:00
#SBATCH --mail-user=jc33471@uga.edu
#SBATCH --mail-type=BEGIN,END,FAIL
#SBATCH --output=/scratch/jc33471/mcc_paper/simulation_smk/simulation_3.out

total_dir="/scratch/jc33471/mcc_paper/simulation_smk"
mkdir -p ${total_dir}
mcc_dir="/home/${USER}/mcclintock"
mcc1_dir="/home/${USER}/mcclintock_1.0/mcclintock/"

CONDA_BASE=$(conda info --base)
source ${CONDA_BASE}/etc/profile.d/conda.sh
conda activate mcc_sim

############### simulation 3 ###############
sim_dir=${total_dir}/sim_3
mkdir -p $sim_dir

cd $sim_dir
mkdir -p logs/ config/

# absolute path to config file for the entire simulation framework
config=${sim_dir}/config/sim_3.json

# make json config for snakemake
python ${mcc_dir}/simulation/make_snakemake_config.py \
    --family "TY1" "TY2" "TY3" "TY4" \
    --bed "${mcc_dir}/simulation/yeast/800-50_tRNA_targets.bed" \
        "${mcc_dir}/simulation/yeast/800-50_tRNA_targets.bed" \
        "${mcc_dir}/simulation/yeast/17-12_tRNA_targets.bed" \
        "${mcc_dir}/simulation/yeast/800-50_tRNA_targets.bed" \
    --mcc ${mcc_dir} \
    --out ${config} \
    --outdir ${sim_dir} \
    --reference "${mcc_dir}/test/sacCer2.fasta" \
    --consensus "${mcc_dir}/test/sac_cer_TE_seqs.fasta" \
    --locations "${mcc_dir}/test/reference_TE_locations.gff" \
    --taxonomy "${mcc_dir}/test/sac_cer_te_families.tsv" \
    --covrange 3 6 12 25 50 100 --numrep 300

snakemake \
    --jobs 200 \
    --use-conda \
    --latency-wait 60 \
    --keep-going \
    --restart-times 3 \
    --snakefile "${mcc_dir}/simulation/Snakefile_sim" \
    --configfile ${config} \
    --cluster '
        sbatch \
            --partition=batch \
            --nodes=1 \
            --tasks-per-node={threads} \
            --mem={resources.mem} \
            --time=24:00:00 \
            --mail-user=jc33471@uga.edu \
            --mail-type=END,FAIL \
            --output=logs/{wildcards.cov}x_{wildcards.strand}_rep{wildcards.rep}.o \
            --error=logs/{wildcards.cov}x_{wildcards.strand}_rep{wildcards.rep}.e'

snakemake \
    --cores $SLURM_NTASKS \
    --use-conda \
    --configfile ${config} \
    --snakefile "${mcc_dir}/simulation/Snakefile_analysis"

rm ${sim_dir}/*/data/*/*.fastq
rm ${sim_dir}/*/data/*/*.sam
```

### Simulation 4
```bash
#!/bin/bash
#SBATCH --partition=iob_p
#SBATCH --job-name=simulation_batch4
#SBATCH --nodes=1
#SBATCH --tasks-per-node=10
#SBATCH --mem=40G
#SBATCH --time=400:00:00
#SBATCH --mail-user=jc33471@uga.edu
#SBATCH --mail-type=BEGIN,END,FAIL
#SBATCH --output=/scratch/jc33471/mcc_paper/simulation_smk/simulation_4.out

total_dir="/scratch/jc33471/mcc_paper/simulation_smk"
mkdir -p ${total_dir}
mcc_dir="/home/${USER}/mcclintock"
mcc1_dir="/home/${USER}/mcclintock_1.0/mcclintock/"

CONDA_BASE=$(conda info --base)
source ${CONDA_BASE}/etc/profile.d/conda.sh
conda activate mcc_sim

############### simulation 4 ###############
sim_dir=${total_dir}/sim_4
mkdir -p $sim_dir

cd $sim_dir
mkdir -p logs/ config/

# absolute path to config file for the entire simulation framework
config=${sim_dir}/config/sim_4.json

# make json config for snakemake
python ${mcc_dir}/simulation/make_snakemake_config.py \
    --family "TY1" "TY2" "TY3" "TY4" \
    --bed "${mcc_dir}/simulation/yeast/non_TE_targets.bed" \
    --mcc ${mcc_dir} \
    --out ${config} \
    --outdir ${sim_dir} \
    --reference "${mcc_dir}/test/sacCer2.fasta" \
    --consensus "${mcc_dir}/test/sac_cer_TE_seqs.fasta" \
    --locations "${mcc_dir}/test/reference_TE_locations.gff" \
    --taxonomy "${mcc_dir}/test/sac_cer_te_families.tsv" \
    --covrange 3 6 12 25 50 100 --numrep 300

snakemake \
    --jobs 200 \
    --use-conda \
    --latency-wait 60 \
    --keep-going \
    --restart-times 3 \
    --snakefile "${mcc_dir}/simulation/Snakefile_sim" \
    --configfile ${config} \
    --cluster '
        sbatch \
            --partition=batch \
            --nodes=1 \
            --tasks-per-node={threads} \
            --mem={resources.mem} \
            --time=24:00:00 \
            --mail-user=jc33471@uga.edu \
            --mail-type=END,FAIL \
            --output=logs/{wildcards.cov}x_{wildcards.strand}_rep{wildcards.rep}.o \
            --error=logs/{wildcards.cov}x_{wildcards.strand}_rep{wildcards.rep}.e'

snakemake \
    --cores $SLURM_NTASKS \
    --use-conda \
    --configfile ${config} \
    --snakefile "${mcc_dir}/simulation/Snakefile_analysis"

rm ${sim_dir}/*/data/*/*.fastq
rm ${sim_dir}/*/data/*/*.sam
```


## Gathering simulation results from batches
```bash
total_dir="/scratch/${USER}/mcc_paper/simulation_smk"
sim1_dir=${total_dir}/sim_1
sim2_dir=${total_dir}/sim_2
sim3_dir=${total_dir}/sim_3
sim4_dir=${total_dir}/sim_4
paper_dir="/home/${USER}/mcclintock2_paper"

conda activate mcc_analysis
Rscript --vanilla ${paper_dir}/simulation_batches/sim_batches_fig.R \
    ${paper_dir} ${sim1_dir} ${sim2_dir} ${sim3_dir} ${sim4_dir} ${total_dir}/final
```

## Inspecting intersections between methods using upset plot
```bash
total_dir="/scratch/${USER}/mcc_paper/simulation_smk"
sim1_dir=${total_dir}/sim_1
sim2_dir=${total_dir}/sim_2
sim3_dir=${total_dir}/sim_3
sim4_dir=${total_dir}/sim_4
paper_dir="/home/${USER}/mcclintock2_paper"

conda activate mcc_sim
for cov in 3 6 12 25 50 100; do
    echo $cov
    python ${paper_dir}/simulation_batches/upset_plot_data.py \
        -o ${sim3_dir}/${cov}

    python ${paper_dir}/simulation_batches/upset_plot_data.py \
        -o ${sim4_dir}/${cov}

done
conda deactivate

conda activate mcc_analysis
Rscript --vanilla ${paper_dir}/simulation_batches/sim_batches_upset.R \
    ${paper_dir} ${sim1_dir} ${sim2_dir} ${sim3_dir} ${sim4_dir} ${total_dir}/final
```